# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.

from kafka.tools.assigner.actions.balancemodules.size import ActionBalanceSize
from kafka.tools.assigner.actions import ActionBalanceModule


class ActionBalanceRate(ActionBalanceModule):
    name = "rate"
    helpstr = "Move the busiest partitions in the cluster to even the total bytes rate per-broker for each replica position"

    def __init__(self, args, cluster):
        super(ActionBalanceRate, self).__init__(args, cluster)

        # This module merely calls the balance "size" module with the scaled_size attr, so we're going to instantiate a copy of that
        self._size = ActionBalanceSize(args, cluster, size_attr='scaled_size')

    def process_cluster(self):
        # Call the size module
        self._size.process_cluster()
