# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.


class PluginModule:
    def __init__(self):
        pass

    def set_default_arguments(self, argument_parser):
        pass

    def set_arguments(self, arguments):
        pass

    def set_cluster(self, cluster):
        pass

    def after_sizes(self):
        pass

    def set_new_cluster(self, cluster):
        pass

    def set_batches(self, batches):
        pass

    def before_execute_batch(self, num):
        pass

    def after_execute_batch(self, num):
        pass

    def before_ple(self):
        pass

    def finished(self):
        pass
